#!/usr/bin/perl

#PanLogURLtoMySQL.pl
#Copyright 2011,2013 Mark Stingley (mark _AT_ altsec.info)
#Licensed under http://www.gnu.org/copyleft/gpl.html
#Original multi-purpose Palo Alto log converter code modified for just URL logs
#Designed to work with "scp export url" or "scheduled export" URL logs

#Refer to PanLogURLtoMySQLschema.txt for the MySQL table schema and table creation statement.

#WARNING: Since this code uses a MySQL login with read/write/create privileges
#for the table, the program file should be in a restricted directory and have
#700 permission, possibly 770 for controlled group execution

#Use the Database independent interface for Perl 
use DBI;

#If $debug is set to Non-Zero, it will write all records to a .csv text file
#in /tmp/ReadPanLogs-debug.csv
$debug = 0;
$OUTfile = "/tmp/ReadPanLogs-debug.csv";


#Check to see if the filename passed as an argument actually exists
if ( ! -e $ARGV[0] ) {
    print "File: $ARGV[0] does not exist... exiting.\n";
    exit 1;
}

#Store the file/path name to $INfile and $_ for processing
$INfile = "$ARGV[0]";
$_ = $INfile;

#Here's the tricky part. This code grabs the date out of the formatted PanLog
#file name; Hostname_url_YYYY_MM_DD_last_calendar_day.csv
#If Palo Alto changes the export filename format, or if your hostname has any
#UNDERSCORES "_", it will need to be adjusted 
#For example: DC-5060_url_2013_09_01_last_calendar_day.csv

#Split the filename into all the needed parts using "_" as a delimiter
($dev,$log,$yr,$mo,$da,$last,$calendar,$daystr)=split("_",$INfile);

#Here the MySQL table name and path arguments are constructed
#Example: database "panlogs", table name "20130901"
#Also note... these are previous day logs. 20130901 contains 2013-08-31 logs
$TblName="$yr$mo$da";
$TblPath="/var/lib/mysql/panlogs/$TblName.MYD";

#Set the username and password for the MySQL database access
$DBuser='PUT_MySQL_USER_NAME_HERE';
$DBpasswd='PUT_MySQL_USER_PASSWORD_HERE';

#open the connection to the database
$panlogs_dbh = DBI->connect('dbi:mysql:panlogs',$DBuser,$DBpasswd) || $DBI::errstr;

#create the table, if it doesn't already exist
if ( ! -e $TblPath ) {
   $sql_ins = "create table panlogs.$TblName like combined_template";
   $ins_sth = $panlogs_dbh->prepare($sql_ins);
   $ins_sth->execute || die $panlogs_dbh->errstr;
}

#If $debug is NonZero, create the output text file
if ( $debug eq 1 ) {

   open (OUTFILE, ">$OUTfile")  || die "Can't open $OUTfile for writing.\n";

}

#Open the Palo Alto URL log export
open (INFILE, "<$INfile")  || die "Can't open $INfile for reading.\n";
#discard the header line
$header = <INFILE>;

#prepare the SQL statement here with placeholders for performance
$ins_sth = $panlogs_dbh->prepare("insert delayed into panlogs.$TblName (domain,rcvtime,sernum,type,subtype,config,gentime,srcaddr,dstaddr,nsrcaddr,ndstaddr,rule,srcuser,dstuser,app,vsys,srczone,dstzone,ibif,obif,logact,logtime,sessid,rptcnt,srcport,dstport,nsrcport,ndstport,flags,proto,action,bytes,bsent,brecv,packets,stime,etime,category,padding,miscurl,threat,severity,direction,seqno,actflags,srccntry,dstcntry,cpadding,contype) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)") or die $panlogs_dbh->errstr;

#Read the data records
while (<INFILE>) {

   chomp;


#This eval ensures that only records with a properly formatted URL, i.e. one
#surrounded by double-quotes are processed

   if ( /\,\"(.+?)\"\,/ ) {

#Break the line up with a simple RegEx. Experience showed that some URL records
#had a crapload of commas. Early work using RegEx to parse the entire record
#often blew up AND was EXTREMELY SLOW.

      ($Beginning,$URL,$Ending) = /^(.*)\"(.*)\"(.*)$/;

#Split the first part of the record into data using a comma delimiter
      ($Domain,$Receive_Time,$Serial_Number,$Type,$Threat_Content_Type,$Config_Version,$Generate_Time,$Source_address,$Destination_address,$NAT_Source_IP,$NAT_Destination_IP,$Rule,$Source_User,$Destination_User,$Application,$Virtual_System,$Source_Zone,$Destination_Zone,$Inbound_Interface,$Outbound_Interface,$Log_Action,$Time_Logged,$Session_ID,$Repeat_Count,$Source_Port,$Destination_Port,$NAT_Source_Port,$NAT_Destination_Port,$Flags,$IP_Protocol,$Action,$Empty) = split(',',$Beginning);

#Split the end section of the record into data using a comma delimiter
($Blank,$Threat_Content_Name,$Category,$Severity,$Direction,$Seqno,$Actflags,$Srccntry,$Dstcntry,$Cpadding,$Contype) = split(',',$Ending);

#Ensure any embedded double-quotes are escaped: \"
   $URL =~ s/"/\\"/g;

#If $debug is NonZero, write the assembled data to the debug .csv text file
   if ( $debug eq 1 ) {
print OUTFILE "$Domain,$Receive_Time,$Serial_Number,$Type,$Threat_Content_Type,$Config_Version,$Generate_Time,$Source_address,$Destination_address,$NAT_Source_IP,$NAT_Destination_IP,$Rule,$Source_User,$Destination_User,$Application,$Virtual_System,$Source_Zone,$Destination_Zone,$Inbound_Interface,$Outbound_Interface,$Log_Action,$Time_Logged,$Session_ID,$Repeat_Count,$Source_Port,$Destination_Port,$NAT_Source_Port,$NAT_Destination_Port,$Flags,$IP_Protocol,$Action,$URL,$Threat_Content_Name,$Category,$Severity,$Direction,$Seqno,$Actflags,$Srccntry,$Dstcntry,$Cpadding,$Contype\n";

   }

#Insert the assembled data into the MySQL table
   $ins_sth->execute($Domain,"$Receive_Time","$Serial_Number","$Type","$Threat_Content_Type",$Config_Version,"$Generate_Time","$Source_address","$Destination_address","$NAT_Source_IP","$NAT_Destination_IP","$Rule","$Source_User","$Destination_User","$Application","$Virtual_System","$Source_Zone","$Destination_Zone","$Inbound_Interface","$Outbound_Interface","$Log_Action","$Time_Logged",$Session_ID,$Repeat_Count,$Source_Port,$Destination_Port,$NAT_Source_Port,$NAT_Destination_Port,"$Flags","$IP_Protocol","$Action",undef,undef,undef,undef,undef,undef,"$Category",undef,"$URL","$Threat_Content_Name","$Severity","$Direction",$Seqno,"$Actflags","$Srccntry","$Dstcntry",$Cpadding,"$Contype") || die $panlogs_dbh->errstr;

   }

}

#After the input file has been parsed, flush the table & disconnect the db
$sql_ins = "flush tables panlogs.$TblName";
$ins_sth = $panlogs_dbh->prepare($sql_ins);
$ins_sth->execute || die $panlogs_dbh->errstr;
$panlogs_dbh->disconnect;

#Close the input and output files
close (INFILE);
if ( $debug eq 1 ) {
   close (OUTFILE);
}


